/*
** $Id: lcomplex.c v 0.1, 15.06.2008/18:56:41 $
** Agena Complex Numbers
** See Copyright Notice in lua.h
*/


/*
** Implementation of Complex Numbers
*/

#define lcomplex_c
#define LUA_CORE

#include "agena.h"

#include "ldebug.h"
#include "ldo.h"
#include "lgc.h"
#include "lmem.h"
#include "lobject.h"
#include "lstate.h"
#include "lcomplex.h"

/* #include <stdio.h> */ /* for fprintf and printf */

#ifndef LUA_ANSI
#include <complex.h>

/*
** }=============================================================
*/


/* create a new complex value */

Complex *agnCmplx_create (lua_State *L, agn_Complex c) {
  Complex *t;
  t = luaM_new(L, Complex);  /* using a specialized newComplex mem function is slower */
  luaC_link(L, obj2gco(t), LUA_TCOMPLEX);
  t->c = c;
  return t;
}


Complex *agnCmplx_make (lua_State *L, const TValue *re, const TValue *im) {
  Complex *t = luaM_new(L, Complex);
  luaC_link(L, obj2gco(t), LUA_TCOMPLEX);
  if (ttisnumber(re) && ttisnumber(im)) {
    t->c = nvalue(re)+nvalue(im)*I;
  } else
    t->c = AGN_NAN;
  return t;
}

#else

/* create a new complex value */

Complex *agnCmplx_create (lua_State *L, lua_Number re, lua_Number im) {
  Complex *t;
  t = luaM_new(L, Complex);  /* using a specialized newComplex mem function is slower */
  luaC_link(L, obj2gco(t), LUA_TCOMPLEX);
  t->re = re;
  t->im = im ;
  return t;
}


Complex *agnCmplx_make (lua_State *L, const TValue *re, const TValue *im) {
  Complex *t = luaM_new(L, Complex);
  luaC_link(L, obj2gco(t), LUA_TCOMPLEX);
  if (ttisnumber(re) && ttisnumber(im)) {
    t->re = nvalue(re);
    t->im = nvalue(im);
  } else {
    t->re = AGN_NAN;
    t->im = AGN_NAN;
  }
  return t;
}

#endif

/* free a complex value */

void agnCmplx_free (lua_State *L, Complex *t) {
  luaM_free(L, t);
}

